use std::path::{Path, PathBuf};

use figment::{
    providers::{Env, Format, Toml},
    Figment,
};
use serde::Deserialize;

#[derive(Deserialize, Debug)]
#[serde(rename_all = "lowercase")]
#[serde(tag = "type")]
pub enum FsConfig {
    Local { data_dir: PathBuf },
}

#[derive(Deserialize, Debug)]
#[serde(rename_all = "lowercase")]
#[serde(tag = "type")]
pub enum DbConfig {
    Sqlite {
        db_dir: PathBuf,
    },
    Postgres {
        host: String,
        user: String,
        password: String,
    },
}

#[derive(Deserialize, Debug)]
pub struct Config {
    api_key: String,
    port: u16,
    log_level: String,
    fs: FsConfig,
    db: DbConfig,
}

impl Config {
    pub fn figment(config_file: impl AsRef<Path>) -> Figment {
        Figment::new()
            .merge(Toml::file(config_file))
            .merge(Env::prefixed("RIETER_"))
    }
}
